@@ -2,20 +2,11 @@ import importlib
 import sys
 import traceback
 
+from tornado.gen import multi
+from traitlets import Any, Bool, Dict, HasTraits, Instance, Unicode, default, observe
+from traitlets import validate as validate_trait
 from traitlets.config import LoggingConfigurable
 
-from traitlets import (
-    HasTraits,
-    Dict,
-    Unicode,
-    Bool,
-    Any,
-    Instance,
-    default,
-    observe,
-    validate,
-)
-
 from .config import ExtensionConfigManager
 from .utils import (
     ExtensionMetadataError,
@@ -29,21 +20,21 @@ class ExtensionPoint(HasTraits):
     """A simple API for connecting to a Jupyter Server extension
     point defined by metadata and importable from a Python package.
     """
+
     _linked = Bool(False)
     _app = Any(None, allow_none=True)
 
     metadata = Dict()
 
-    @validate('metadata')
+    @validate_trait("metadata")
     def _valid_metadata(self, proposed):
-        metadata = proposed['value']
+        metadata = proposed["value"]
         # Verify that the metadata has a "name" key.
         try:
-            self._module_name = metadata['module']
+            self._module_name = metadata["module"]
         except KeyError:
             raise ExtensionMetadataError(
-                "There is no 'module' key in the extension's "
-                "metadata packet."
+                "There is no 'module' key in the extension's metadata packet."
             )
 
         try:
@@ -54,7 +45,7 @@ class ExtensionPoint(HasTraits):
                 "sure the extension is installed?".format(self._module_name)
             )
         # If the metadata includes an ExtensionApp, create an instance.
-        if 'app' in metadata:
+        if "app" in metadata:
             self._app = metadata["app"]()
         return metadata
 
@@ -104,8 +95,7 @@ class ExtensionPoint(HasTraits):
 
     @property
     def module(self):
-        """The imported module (using importlib.import_module)
-        """
+        """The imported module (using importlib.import_module)"""
         return self._module
 
     def _get_linker(self):
@@ -115,9 +105,9 @@ class ExtensionPoint(HasTraits):
             linker = getattr(
                 self.module,
                 # Search for a _link_jupyter_extension
-                '_link_jupyter_server_extension',
+                "_link_jupyter_server_extension",
                 # Otherwise return a dummy function.
-                lambda serverapp: None
+                lambda serverapp: None,
             )
         return linker
 
@@ -168,6 +158,7 @@ class ExtensionPackage(HasTraits):
     ext_name = "my_extensions"
     extpkg = ExtensionPackage(name=ext_name)
     """
+
     name = Unicode(help="Name of the an importable Python package.")
     enabled = Bool(False).tag(config=True)
 
@@ -176,11 +167,11 @@ class ExtensionPackage(HasTraits):
         self._linked_points = {}
         super().__init__(*args, **kwargs)
 
-    _linked_points = {}
+    _linked_points: dict = {}
 
-    @validate("name")
+    @validate_trait("name")
     def _validate_name(self, proposed):
-        name = proposed['value']
+        name = proposed["value"]
         self._extension_points = {}
         try:
             self._module, self._metadata = get_metadata(name)
@@ -230,15 +221,14 @@ class ExtensionPackage(HasTraits):
 
     def load_point(self, point_name, serverapp):
         point = self.extension_points[point_name]
-        point.load(serverapp)
+        return point.load(serverapp)
 
     def link_all_points(self, serverapp):
         for point_name in self.extension_points:
             self.link_point(point_name, serverapp)
 
     def load_all_points(self, serverapp):
-        for point_name in self.extension_points:
-            self.load_point(point_name, serverapp)
+        return [self.load_point(point_name, serverapp) for point_name in self.extension_points]
 
 
 class ExtensionManager(LoggingConfigurable):
@@ -251,6 +241,8 @@ class ExtensionManager(LoggingConfigurable):
 
     config_manager = Instance(ExtensionConfigManager, allow_none=True)
 
+    serverapp = Any()  # Use Any to avoid circular import of Instance(ServerApp)
+
     @default("config_manager")
     def _load_default_config_manager(self):
         config_manager = ExtensionConfigManager()
@@ -273,6 +265,11 @@ class ExtensionManager(LoggingConfigurable):
         """
     )
 
+    @property
+    def sorted_extensions(self):
+        """Returns an extensions dictionary, sorted alphabetically."""
+        return dict(sorted(self.extensions.items()))
+
     # The `_linked_extensions` attribute tracks when each extension
     # has been successfully linked to a ServerApp. This helps prevent
     # extensions from being re-linked recursively unintentionally if another
@@ -285,12 +282,20 @@ class ExtensionManager(LoggingConfigurable):
         """
     )
 
+    @property
+    def extension_apps(self):
+        """Return mapping of extension names and sets of ExtensionApp objects."""
+        return {
+            name: {point.app for point in extension.extension_points.values() if point.app}
+            for name, extension in self.extensions.items()
+        }
+
     @property
     def extension_points(self):
-        extensions = self.extensions
+        """Return mapping of extension point names and ExtensionPoint objects."""
         return {
             name: point
-            for value in extensions.values()
+            for value in self.extensions.values()
             for name, point in value.extension_points.items()
         }
 
@@ -319,45 +324,73 @@ class ExtensionManager(LoggingConfigurable):
             return True
         # Raise a warning if the extension cannot be loaded.
         except Exception as e:
+            if self.serverapp.reraise_server_extension_failures:
+                raise
             self.log.warning(e)
         return False
 
-    def link_extension(self, name, serverapp):
+    def link_extension(self, name):
         linked = self.linked_extensions.get(name, False)
         extension = self.extensions[name]
         if not linked and extension.enabled:
             try:
                 # Link extension and store links
-                extension.link_all_points(serverapp)
+                extension.link_all_points(self.serverapp)
                 self.linked_extensions[name] = True
-                self.log.info("{name} | extension was successfully linked.".format(name=name))
+                self.log.info(f"{name} | extension was successfully linked.")
             except Exception as e:
+                if self.serverapp.reraise_server_extension_failures:
+                    raise
                 self.log.warning(e)
 
-    def load_extension(self, name, serverapp):
+    def load_extension(self, name):
         extension = self.extensions.get(name)
+
         if extension.enabled:
             try:
-                extension.load_all_points(serverapp)
-                self.log.info("{name} | extension was successfully loaded.".format(name=name))
+                extension.load_all_points(self.serverapp)
             except Exception as e:
+                if self.serverapp.reraise_server_extension_failures:
+                    raise
                 self.log.debug("".join(traceback.format_exception(*sys.exc_info())))
-                self.log.warning("{name} | extension failed loading with message: {error}".format(name=name,error=str(e)))
+                self.log.warning(
+                    "{name} | extension failed loading with message: {error}".format(
+                        name=name, error=str(e)
+                    )
+                )
+            else:
+                self.log.info(f"{name} | extension was successfully loaded.")
 
-    def link_all_extensions(self, serverapp):
+    async def stop_extension(self, name, apps):
+        """Call the shutdown hooks in the specified apps."""
+        for app in apps:
+            self.log.debug(f'{name} | extension app "{app.name}" stopping')
+            await app.stop_extension()
+            self.log.debug(f'{name} | extension app "{app.name}" stopped')
+
+    def link_all_extensions(self):
         """Link all enabled extensions
         to an instance of ServerApp
         """
         # Sort the extension names to enforce deterministic linking
         # order.
-        for name in sorted(self.extensions.keys()):
-            self.link_extension(name, serverapp)
+        for name in self.sorted_extensions.keys():
+            self.link_extension(name)
 
-    def load_all_extensions(self, serverapp):
+    def load_all_extensions(self):
         """Load all enabled extensions and append them to
         the parent ServerApp.
         """
         # Sort the extension names to enforce deterministic loading
         # order.
-        for name in sorted(self.extensions.keys()):
-            self.load_extension(name, serverapp)
+        for name in self.sorted_extensions.keys():
+            self.load_extension(name)
+
+    async def stop_all_extensions(self):
+        """Call the shutdown hooks in all extensions."""
+        await multi(
+            [
+                self.stop_extension(name, apps)
+                for name, apps in sorted(dict(self.extension_apps).items())
+            ]
+        )
