@@ -6,7 +6,7 @@ import os
 from logging import Logger
 from queue import Queue
 from threading import Thread
-from typing import Dict
+from typing import Any, Dict, Optional
 
 import websocket
 from jupyter_client.asynchronous.client import AsyncKernelClient
@@ -15,22 +15,14 @@ from jupyter_client.kernelspec import KernelSpecManager
 from jupyter_client.manager import AsyncKernelManager
 from jupyter_client.managerabc import KernelManagerABC
 from tornado import web
-from tornado.escape import json_decode
-from tornado.escape import json_encode
-from tornado.escape import url_escape
-from tornado.escape import utf8
-from traitlets import default
-from traitlets import DottedObjectName
-from traitlets import Instance
-from traitlets import Type
+from tornado.escape import json_decode, json_encode, url_escape, utf8
+from traitlets import DottedObjectName, Instance, Type, default
 
 from .._tz import UTC
 from ..services.kernels.kernelmanager import AsyncMappingKernelManager
 from ..services.sessions.sessionmanager import SessionManager
-from ..utils import ensure_async
-from ..utils import url_path_join
-from .gateway_client import gateway_request
-from .gateway_client import GatewayClient
+from ..utils import ensure_async, url_path_join
+from .gateway_client import GatewayClient, gateway_request
 
 
 class GatewayMappingKernelManager(AsyncMappingKernelManager):
@@ -80,7 +72,7 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
                 kwargs["cwd"] = self.cwd_for_path(path)
 
         km = self.kernel_manager_factory(parent=self, log=self.log)
-        await km.start_kernel(**kwargs)
+        await km.start_kernel(kernel_id=kernel_id, **kwargs)
         kernel_id = km.kernel_id
         self._kernels[kernel_id] = km
 
@@ -125,9 +117,11 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
         # Remove any of our kernels that may have been culled on the gateway server
         our_kernels = self._kernels.copy()
         culled_ids = []
-        for kid, km in our_kernels.items():
+        for kid, _ in our_kernels.items():
             if kid not in kernel_models:
-                self.log.warn(f"Kernel {kid} no longer active - probably culled on Gateway server.")
+                self.log.warning(
+                    f"Kernel {kid} no longer active - probably culled on Gateway server."
+                )
                 self._kernels.pop(kid, None)
                 culled_ids.append(kid)  # TODO: Figure out what do with these.
         return list(kernel_models.values())
@@ -374,6 +368,7 @@ class GatewayKernelManager(AsyncKernelManager):
             self.log.debug("Request kernel at: %s" % self.kernel_url)
             try:
                 response = await gateway_request(self.kernel_url, method="GET")
+
             except web.HTTPError as error:
                 if error.status_code == 404:
                     self.log.warning("Kernel not found at: %s" % self.kernel_url)
@@ -440,15 +435,13 @@ class GatewayKernelManager(AsyncKernelManager):
             response = await gateway_request(self.kernels_url, method="POST", body=json_body)
             self.kernel = json_decode(response.body)
             self.kernel_id = self.kernel["id"]
-            self.log.info(
-                "GatewayKernelManager started kernel: {}, args: {}".format(self.kernel_id, kwargs)
-            )
+            self.kernel_url = url_path_join(self.kernels_url, url_escape(str(self.kernel_id)))
+            self.log.info(f"GatewayKernelManager started kernel: {self.kernel_id}, args: {kwargs}")
         else:
             self.kernel_id = kernel_id
+            self.kernel_url = url_path_join(self.kernels_url, url_escape(str(self.kernel_id)))
             self.kernel = await self.refresh_model()
-            self.log.info("GatewayKernelManager using existing kernel: {}".format(self.kernel_id))
-
-        self.kernel_url = url_path_join(self.kernels_url, url_escape(str(self.kernel_id)))
+            self.log.info(f"GatewayKernelManager using existing kernel: {self.kernel_id}")
 
     async def shutdown_kernel(self, now=False, restart=False):
         """Attempts to stop the kernel process cleanly via HTTP."""
@@ -461,6 +454,7 @@ class GatewayKernelManager(AsyncKernelManager):
     async def restart_kernel(self, **kw):
         """Restarts a kernel via HTTP."""
         if self.has_kernel:
+            assert self.kernel_url is not None
             kernel_url = self.kernel_url + "/restart"
             self.log.debug("Request restart kernel at: %s", kernel_url)
             response = await gateway_request(kernel_url, method="POST", body=json_encode({}))
@@ -469,6 +463,7 @@ class GatewayKernelManager(AsyncKernelManager):
     async def interrupt_kernel(self):
         """Interrupts the kernel via an HTTP request."""
         if self.has_kernel:
+            assert self.kernel_url is not None
             kernel_url = self.kernel_url + "/interrupt"
             self.log.debug("Request interrupt kernel at: %s", kernel_url)
             response = await gateway_request(kernel_url, method="POST", body=json_encode({}))
@@ -493,7 +488,7 @@ KernelManagerABC.register(GatewayKernelManager)
 
 class ChannelQueue(Queue):
 
-    channel_name: str = None
+    channel_name: Optional[str] = None
 
     def __init__(self, channel_name: str, channel_socket: websocket.WebSocket, log: Logger):
         super().__init__()
@@ -501,7 +496,7 @@ class ChannelQueue(Queue):
         self.channel_socket = channel_socket
         self.log = log
 
-    async def get_msg(self, *args, **kwargs) -> dict:
+    async def get_msg(self, *args: Any, **kwargs: Any) -> dict:
         timeout = kwargs.get("timeout", 1)
         msg = self.get(timeout=timeout)
         self.log.debug(
@@ -523,7 +518,7 @@ class ChannelQueue(Queue):
 
     @staticmethod
     def serialize_datetime(dt):
-        if isinstance(dt, (datetime.date, datetime.datetime)):
+        if isinstance(dt, (datetime.datetime)):
             return dt.timestamp()
         return None
 
@@ -579,13 +574,18 @@ class GatewayKernelClient(AsyncKernelClient):
     # flag for whether execute requests should be allowed to call raw_input:
     allow_stdin = False
     _channels_stopped = False
-    _channel_queues = {}
+    _channel_queues: Optional[dict] = {}
+    _control_channel: Optional[ChannelQueue]
+    _hb_channel: Optional[ChannelQueue]
+    _stdin_channel: Optional[ChannelQueue]
+    _iopub_channel: Optional[ChannelQueue]
+    _shell_channel: Optional[ChannelQueue]
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
         self.kernel_id = kwargs["kernel_id"]
-        self.channel_socket = None
-        self.response_router = None
+        self.channel_socket: Optional[websocket.WebSocket] = None
+        self.response_router: Optional[Thread] = None
 
     # --------------------------------------------------------------------------
     # Channel management methods
@@ -606,7 +606,7 @@ class GatewayKernelClient(AsyncKernelClient):
             "channels",
         )
         # Gather cert info in case where ssl is desired...
-        ssl_options = dict()
+        ssl_options = {}
         ssl_options["ca_certs"] = GatewayClient.instance().ca_certs
         ssl_options["certfile"] = GatewayClient.instance().client_cert
         ssl_options["keyfile"] = GatewayClient.instance().client_key
@@ -634,7 +634,9 @@ class GatewayKernelClient(AsyncKernelClient):
         self._channels_stopped = True
         self.log.debug("Closing websocket connection")
 
+        assert self.channel_socket is not None
         self.channel_socket.close()
+        assert self.response_router is not None
         self.response_router.join()
 
         if self._channel_queues:
@@ -648,7 +650,9 @@ class GatewayKernelClient(AsyncKernelClient):
         """Get the shell channel object for this kernel."""
         if self._shell_channel is None:
             self.log.debug("creating shell channel queue")
+            assert self.channel_socket is not None
             self._shell_channel = ChannelQueue("shell", self.channel_socket, self.log)
+            assert self._channel_queues is not None
             self._channel_queues["shell"] = self._shell_channel
         return self._shell_channel
 
@@ -657,7 +661,9 @@ class GatewayKernelClient(AsyncKernelClient):
         """Get the iopub channel object for this kernel."""
         if self._iopub_channel is None:
             self.log.debug("creating iopub channel queue")
+            assert self.channel_socket is not None
             self._iopub_channel = ChannelQueue("iopub", self.channel_socket, self.log)
+            assert self._channel_queues is not None
             self._channel_queues["iopub"] = self._iopub_channel
         return self._iopub_channel
 
@@ -666,7 +672,9 @@ class GatewayKernelClient(AsyncKernelClient):
         """Get the stdin channel object for this kernel."""
         if self._stdin_channel is None:
             self.log.debug("creating stdin channel queue")
+            assert self.channel_socket is not None
             self._stdin_channel = ChannelQueue("stdin", self.channel_socket, self.log)
+            assert self._channel_queues is not None
             self._channel_queues["stdin"] = self._stdin_channel
         return self._stdin_channel
 
@@ -675,7 +683,9 @@ class GatewayKernelClient(AsyncKernelClient):
         """Get the hb channel object for this kernel."""
         if self._hb_channel is None:
             self.log.debug("creating hb channel queue")
+            assert self.channel_socket is not None
             self._hb_channel = HBChannelQueue("hb", self.channel_socket, self.log)
+            assert self._channel_queues is not None
             self._channel_queues["hb"] = self._hb_channel
         return self._hb_channel
 
@@ -684,7 +694,9 @@ class GatewayKernelClient(AsyncKernelClient):
         """Get the control channel object for this kernel."""
         if self._control_channel is None:
             self.log.debug("creating control channel queue")
+            assert self.channel_socket is not None
             self._control_channel = ChannelQueue("control", self.channel_socket, self.log)
+            assert self._channel_queues is not None
             self._channel_queues["control"] = self._control_channel
         return self._control_channel
 
@@ -698,11 +710,13 @@ class GatewayKernelClient(AsyncKernelClient):
         """
         try:
             while not self._channels_stopped:
+                assert self.channel_socket is not None
                 raw_message = self.channel_socket.recv()
                 if not raw_message:
                     break
                 response_message = json_decode(utf8(raw_message))
                 channel = response_message["channel"]
+                assert self._channel_queues is not None
                 self._channel_queues[channel].put_nowait(response_message)
 
         except websocket.WebSocketConnectionClosedException:
@@ -710,7 +724,7 @@ class GatewayKernelClient(AsyncKernelClient):
 
         except BaseException as be:
             if not self._channels_stopped:
-                self.log.warning("Unexpected exception encountered ({})".format(be))
+                self.log.warning(f"Unexpected exception encountered ({be})")
 
         self.log.debug("Response router thread exiting...")
 
