@@ -77,7 +77,7 @@ def add_invalid_cell(notebook):
 
 
 async def prepare_notebook(
-    jp_contents_manager, make_invalid: Optional[bool] = False
+    jp_contents_manager: FileContentsManager, make_invalid: Optional[bool] = False
 ) -> Tuple[Dict, str]:
     cm = jp_contents_manager
     model = await ensure_async(cm.new_untitled(type="notebook"))
@@ -259,6 +259,166 @@ async def test_403(jp_file_contents_manager_class, tmp_path):
     except HTTPError as e:
         assert e.status_code == 403
 
+@pytest.mark.skipif(sys.platform.startswith('win'), reason="Can't test hidden files on Windows")
+async def test_400(jp_file_contents_manager_class, tmp_path):
+    #Test Delete behavior
+    #Test delete of file in hidden directory
+    with pytest.raises(HTTPError) as excinfo:
+        td = str(tmp_path)
+        cm = jp_file_contents_manager_class(root_dir=td)
+        hidden_dir = '.hidden'
+        file_in_hidden_path = os.path.join(hidden_dir,'visible.txt')
+        _make_dir(cm, hidden_dir)
+        model = await ensure_async(cm.new(path=file_in_hidden_path))
+        os_path = cm._get_os_path(model['path'])
+
+        try:
+            result = await ensure_async(cm.delete_file(os_path))
+        except HTTPError as e:
+            assert e.status_code == 400
+
+    #Test delete hidden file in visible directory
+    with pytest.raises(HTTPError) as excinfo:
+        td = str(tmp_path)
+        cm = jp_file_contents_manager_class(root_dir=td)
+        hidden_dir = 'visible'
+        file_in_hidden_path = os.path.join(hidden_dir,'.hidden.txt')
+        _make_dir(cm, hidden_dir)
+        model = await ensure_async(cm.new(path=file_in_hidden_path))
+        os_path = cm._get_os_path(model['path'])
+
+        try:
+            result = await ensure_async(cm.delete_file(os_path))
+        except HTTPError as e:
+            assert e.status_code == 400
+
+    #Test Save behavior
+    #Test save of file in hidden directory
+    with pytest.raises(HTTPError) as excinfo:
+        td = str(tmp_path)
+        cm = jp_file_contents_manager_class(root_dir=td)
+        hidden_dir = '.hidden'
+        file_in_hidden_path = os.path.join(hidden_dir,'visible.txt')
+        _make_dir(cm, hidden_dir)
+        model = await ensure_async(cm.new(path=file_in_hidden_path))
+        os_path = cm._get_os_path(model['path'])
+
+        try:
+            result = await ensure_async(cm.save(model,path=os_path))
+        except HTTPError as e:
+            assert e.status_code == 400
+
+    #Test save hidden file in visible directory
+    with pytest.raises(HTTPError) as excinfo:
+        td = str(tmp_path)
+        cm = jp_file_contents_manager_class(root_dir=td)
+        hidden_dir = 'visible'
+        file_in_hidden_path = os.path.join(hidden_dir,'.hidden.txt')
+        _make_dir(cm, hidden_dir)
+        model = await ensure_async(cm.new(path=file_in_hidden_path))
+        os_path = cm._get_os_path(model['path'])
+
+        try:
+            result = await ensure_async(cm.save(model,path=os_path))
+        except HTTPError as e:
+            assert e.status_code == 400
+
+    #Test rename behavior
+    #Test rename with source file in hidden directory
+    with pytest.raises(HTTPError) as excinfo:
+        td = str(tmp_path)
+        cm = jp_file_contents_manager_class(root_dir=td)
+        hidden_dir = '.hidden'
+        file_in_hidden_path = os.path.join(hidden_dir,'visible.txt')
+        _make_dir(cm, hidden_dir)
+        model = await ensure_async(cm.new(path=file_in_hidden_path))
+        old_path = cm._get_os_path(model['path'])
+        new_path = "new.txt"
+
+        try:
+            result = await ensure_async(cm.rename_file(old_path, new_path))
+        except HTTPError as e:
+            assert e.status_code == 400
+
+    #Test rename of dest file in hidden directory
+    with pytest.raises(HTTPError) as excinfo:
+        td = str(tmp_path)
+        cm = jp_file_contents_manager_class(root_dir=td)
+        hidden_dir = '.hidden'
+        file_in_hidden_path = os.path.join(hidden_dir,'visible.txt')
+        _make_dir(cm, hidden_dir)
+        model = await ensure_async(cm.new(path=file_in_hidden_path))
+        new_path = cm._get_os_path(model['path'])
+        old_path = "old.txt"
+
+        try:
+            result = await ensure_async(cm.rename_file(old_path, new_path))
+        except HTTPError as e:
+            assert e.status_code == 400
+
+    #Test rename with hidden source file in visible directory
+    with pytest.raises(HTTPError) as excinfo:
+        td = str(tmp_path)
+        cm = jp_file_contents_manager_class(root_dir=td)
+        hidden_dir = 'visible'
+        file_in_hidden_path = os.path.join(hidden_dir,'.hidden.txt')
+        _make_dir(cm, hidden_dir)
+        model = await ensure_async(cm.new(path=file_in_hidden_path))
+        old_path = cm._get_os_path(model['path'])
+        new_path = "new.txt"
+
+        try:
+            result = await ensure_async(cm.rename_file(old_path, new_path))
+        except HTTPError as e:
+            assert e.status_code == 400
+
+    #Test rename with hidden dest file in visible directory
+    with pytest.raises(HTTPError) as excinfo:
+        td = str(tmp_path)
+        cm = jp_file_contents_manager_class(root_dir=td)
+        hidden_dir = 'visible'
+        file_in_hidden_path = os.path.join(hidden_dir,'.hidden.txt')
+        _make_dir(cm, hidden_dir)
+        model = await ensure_async(cm.new(path=file_in_hidden_path))
+        new_path = cm._get_os_path(model['path'])
+        old_path = "old.txt"
+
+        try:
+            result = await ensure_async(cm.rename_file(old_path, new_path))
+        except HTTPError as e:
+            assert e.status_code == 400
+
+@pytest.mark.skipif(sys.platform.startswith('win'), reason="Can't test hidden files on Windows")
+async def test_404(jp_file_contents_manager_class, tmp_path):
+    #Test visible file in hidden folder
+    with pytest.raises(HTTPError) as excinfo:
+        td = str(tmp_path)
+        cm = jp_file_contents_manager_class(root_dir=td)
+        hidden_dir = '.hidden'
+        file_in_hidden_path = os.path.join(hidden_dir,'visible.txt')
+        _make_dir(cm, hidden_dir)
+        model = await ensure_async(cm.new(path=file_in_hidden_path))
+        os_path = cm._get_os_path(model['path'])
+
+        try:
+            result = await ensure_async(cm.get(os_path, 'w'))
+        except HTTPError as e:
+            assert e.status_code == 404
+
+    #Test hidden file in visible folder
+    with pytest.raises(HTTPError) as excinfo:
+        td = str(tmp_path)
+        cm = jp_file_contents_manager_class(root_dir=td)
+        hidden_dir = 'visible'
+        file_in_hidden_path = os.path.join(hidden_dir,'.hidden.txt')
+        _make_dir(cm, hidden_dir)
+        model = await ensure_async(cm.new(path=file_in_hidden_path))
+        os_path = cm._get_os_path(model['path'])
+
+        try:
+            result = await ensure_async(cm.get(os_path, 'w'))
+        except HTTPError as e:
+            assert e.status_code == 404
 
 async def test_escape_root(jp_file_contents_manager_class, tmp_path):
     td = str(tmp_path)
@@ -756,7 +916,7 @@ async def test_validate_notebook_model(jp_contents_manager):
     with patch("jupyter_server.services.contents.manager.validate_nb") as mock_validate_nb:
         # Valid notebook and a non-None dictionary, no validate call expected
 
-        validation_error = {}
+        validation_error: dict = {}
         cm.validate_notebook_model(model, validation_error)
         assert mock_validate_nb.call_count == 0
         mock_validate_nb.reset_mock()
